// ***************************************************************
// This file was created using the CreateProject.sh script.
// CreateProject.sh is part of Bayesian Analysis Toolkit (BAT).
// BAT can be downloaded from http://www.mppmu.mpg.de/bat
// ***************************************************************

#include "CCXSec.h"

#include <BAT/BCMath.h>

// ---------------------------------------------------------
CCXSec::CCXSec() : BCModel()
{
   // default constructor
   DefineParameters();
};

// ---------------------------------------------------------
CCXSec::CCXSec(const char * name) : BCModel(name)
{
   // constructor
   DefineParameters();
};

// ---------------------------------------------------------
CCXSec::~CCXSec()
   // default destructor
{
};

// ---------------------------------------------------------
void CCXSec::DefineParameters()
{
   // Add parameters to your model here.
   // You can then use them in the methods below by calling the
   // parameters.at(i) or parameters[i], where i is the index
   // of the parameter. The indices increase from 0 according to the
   // order of adding the parameters.

  AddParameter("sig0", 0.0, 50.0); 
  AddParameter("sig1", 0.0, 20.0); 
  AddParameter("dp",  -  0.1,     0.1);
  AddParameter("ds",  -  0.017,   0.017);

}

// ---------------------------------------------------------
double CCXSec::LogLikelihood(const std::vector<double> &parameters)
{
   // This methods returns the logarithm of the conditional probability
   // p(data|parameters). This is where you have to define your model.

   double logprob = 0.;

  // get parameters 
  double sig0 = parameters.at(0); 
  double sig1 = parameters.at(1); 
  double dp   = parameters.at(2);
  double ds   = parameters.at(3); 

  for (int i = 0; i < GetNDataPoints(); ++i)
  {
    // get data point 
    double P      = GetDataPoint(i)->GetValue(0); 
    double sig    = GetDataPoint(i)->GetValue(1); 
    double errsig = GetDataPoint(i)->GetValue(2); 

    // calculate expected cross-section 
    double sigexp = (P * (1 + dp) * (sig0 - sig1) + sig0) * (1 + ds); 

    // calculate likelihood 
    logprob += BCMath::LogGaus(sig, sigexp, errsig);
  }

   return logprob;
}

// ---------------------------------------------------------
double CCXSec::LogAPrioriProbability(const std::vector<double> &parameters)
{
   // This method returns the logarithm of the prior probability for the
   // parameters p(parameters).

   double logprob = 0.;

   // get parameters 
   double sig0 = parameters.at(0);
   double dp   = parameters.at(2);
   
   // use result from measurement with unpolarized beam as prior for
   // parameter sig0
   if (sig0 > 35.)
     logprob += log(sig0 - 35.0) - (sig0 - 35.0) * (sig0 - 35.0) / 2.88; 
   else
     logprob += -999;

   // use prior knowledge of nuisance parameter dp
   logprob += BCMath::LogGaus(dp, 0.0, 0.02);
   
//   double x = parameters.at(0);
//   double y = parameters.at(1);

//   double dx = GetParameter(0)->GetRangeWidth();

//   logprob += log(1./dx);                    // flat prior for x
//   logprob += BCMath::LogGaus(y, 0., 1.0);   // Gaussian prior for y

   return logprob;
}
// ---------------------------------------------------------

